#include <NTPClient.h>
#include <WiFi.h>
#include <WiFiUdp.h>
#include <time.h>
#include <Adafruit_NeoPixel.h>

#define WiFi_led 4
#define dir1 15
#define step1 13
#define dir2 12
#define step2 14
#define MS1 32
#define MS2 33
#define MS3 25
#define lms1 4
#define lms2 2
#define neo 18
#define trigPin 16
#define echoPin 17

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(19, neo, NEO_GRB + NEO_KHZ800);

int HR = 0, MIN = 0, SEC = 0;
int HR_P = 0, MIN_P = 0;
int USHr = 0, USMin = 0, USSec = 0;

const char *ssid     = "panakkal";
const char *password = "JacOb123";

WiFiClient client;

const char* ntpServer   = "pool.ntp.org";

int  gmtOffset_sec_US      = -18000;
int  gmtOffset_sec_IN      = 19800;

char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

WiFiUDP ntpUDP;
NTPClient timeClientIN(ntpUDP, "pool.ntp.org", gmtOffset_sec_IN);
NTPClient timeClientUS(ntpUDP, "pool.ntp.org", gmtOffset_sec_US);

void setup()
{
  pinMode(WiFi_led, OUTPUT);
  pinMode(step1, OUTPUT);
  pinMode(dir1, OUTPUT);
  pinMode(step2, OUTPUT);
  pinMode(dir2, OUTPUT);
  pinMode(MS1, OUTPUT);
  pinMode(MS2, OUTPUT);
  pinMode(MS2, OUTPUT);
  pinMode(neo, OUTPUT);
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(lms1, INPUT_PULLUP);
  pinMode(lms2, INPUT_PULLUP);
  
  Serial.begin(115200);

  WiFi.begin(ssid, password);

  Serial.println();
  Serial.println("Connecting to WiFi");

  while ( WiFi.status() != WL_CONNECTED ) 
  {
    digitalWrite(WiFi_led, HIGH);
    Serial.print( "." );
    delay ( 250 );

    digitalWrite(WiFi_led, LOW);
    Serial.print( "." );
    delay ( 250 );
  }
  
  digitalWrite(WiFi_led, HIGH);
  Serial.println();
  Serial.println("WiFi Connected, Fetching Time from web"); 
  pixels.begin();
  rotate_zero();
  timeClientIN.begin();
  timeClientUS.begin();
  Set_Needle();
}  

void loop() 
{
  UpdateINTime();
  Change_INTime();
  Neo_IN();
  Presence();
  delay(500);
}

void rotate_zero()                                  //Function to set the needles to zero position
{
  int limit1 = digitalRead(lms1);
  int limit2 = digitalRead(lms2);
  digitalWrite(MS1, LOW);
  digitalWrite(MS2, LOW);
  digitalWrite(MS3, LOW);
  digitalWrite(dir1, LOW);
  digitalWrite(dir2, LOW);

  for(int x = 0; x < 3500 && limit1 == 1; x++)
  {
    digitalWrite(step1, HIGH);
    delayMicroseconds(1000);
    digitalWrite(step1, LOW);
    delayMicroseconds(1000);
    limit1 = digitalRead(lms1);
  }
  
  for(int x = 0; x < 3500 && limit2 == 1; x++)
  {
    digitalWrite(step2, HIGH);
    delayMicroseconds(1000);
    digitalWrite(step2, LOW);
    delayMicroseconds(1000);
    limit2 = digitalRead(lms2);
  }  
}

void Set_Needle()                                        //Function to set the needles to the current IST
{
  rotate_zero();
  
  UpdateINTime();
  Serial.print("INDIA      : ");
  Serial.print(daysOfTheWeek[timeClientIN.getDay()]);
  Serial.print(", ");
  Serial.print(HR); 
  Serial.print(":"); 
  Serial.print(MIN);
  Serial.print(":"); 
  Serial.println(SEC);

  UpdateUSTime();

  int stephr = 0;
  int stepmin = 0;

  if(stepmin != MIN)
  {
    digitalWrite(MS1, LOW);
    digitalWrite(MS2, LOW);
    digitalWrite(MS3, LOW);
    
    if(MIN <= 30)
    {
      digitalWrite(dir1, HIGH);
      for(int x = 0; x < (51.2*MIN); x++)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
      }
    }  

    if(MIN > 30)
    {
      digitalWrite(dir1, LOW);
      for(int x = 0; x < (51.2*(60-MIN)); x++)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
      }  
    }
    stepmin = MIN;
  }

  if(stephr != HR)
  {
    digitalWrite(MS1, LOW);
    digitalWrite(MS2, LOW);
    digitalWrite(MS3, LOW);

    if((HR > 0 && HR <= 6) || (HR > 12 && HR <= 18))
    {
      digitalWrite(dir2, HIGH);
      if(HR > 0 && HR <= 6)
      {
        for(int x = 0; x < (256*HR); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }
      }
      else
      {
        for(int x = 0; x < (256*(HR-12)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }  
      }
    }

    if((HR > 6 && HR < 12) || (HR > 18 && HR < 24))
    {
      digitalWrite(dir2, LOW);
      if(HR > 6 && HR < 12)
      {
        for(int x = 0; x < (256*(12-HR)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }
      }
      else
      {
        for(int x = 0; x < (256*(24-HR)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }  
      }
    }  
    stephr = HR;
  }

  MIN_P = MIN;
  HR_P = HR;
}

void UpdateINTime()                                            //Function to fetch IST from internet
{
  timeClientIN.update();
  HR = timeClientIN.getHours();
  MIN = timeClientIN.getMinutes();
  SEC = timeClientIN.getSeconds();   
}

void UpdateUSTime()                                            //Function to fetch EST from internet
{
  timeClientUS.update();
  USHr = timeClientUS.getHours();
  USMin = timeClientUS.getMinutes();
  USSec = timeClientUS.getSeconds();   

  Serial.print("NEW JERSEY : ");
  Serial.print(daysOfTheWeek[timeClientUS.getDay()]);
  Serial.print(", ");
  Serial.print(USHr); 
  Serial.print(":"); 
  Serial.print(USMin);
  Serial.print(":"); 
  Serial.println(USSec);
}

void Change_INTime()                                             //Function to move the needles every minute
{
  int limit1 = digitalRead(lms1);
  int limit2 = digitalRead(lms2);
  
  if(MIN_P != MIN)
  {
    Serial.print("INDIA      : ");
    Serial.print(daysOfTheWeek[timeClientIN.getDay()]);
    Serial.print(", ");
    Serial.print(HR); 
    Serial.print(":"); 
    Serial.print(MIN);
    Serial.print(":"); 
    Serial.println(SEC);

    UpdateUSTime();
    
    digitalWrite(dir1, HIGH);
    if(MIN == 0)
    {
      for(int x = 0; x < 3500 && limit1 == 1; x++)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
        limit1 = digitalRead(lms1);
      }  
    }
    else
    {
      for(int x = 0; x < (51.2*(MIN-MIN_P)); ++x)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
      }
    
      MIN_P = MIN;
    }
  }  

  if(HR_P != HR)
  {
    digitalWrite(dir2, HIGH);
    if(HR == 0 || HR == 12)
    {
      for(int x = 0; x < 3500 && limit2 == 1; x++)
      {
        digitalWrite(step2, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step2, LOW);
        delayMicroseconds(1000);
        limit2 = digitalRead(lms2);
      }
    }
    else
    {
      for(int x = 0; x < (256*(HR-HR_P)); ++x)
      {
        digitalWrite(step2, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step2, LOW);
        delayMicroseconds(1000);
      }
    }  

    HR_P = HR;
  }  
}

void Show_USTime()                                               //Function to move the needles to show the EST
{
  rotate_zero();
  UpdateUSTime();

  int stephr = 0;
  int stepmin = 0;

  if(stepmin != USMin)
  {
    digitalWrite(MS1, LOW);
    digitalWrite(MS2, LOW);
    digitalWrite(MS3, LOW);
    
    if(USMin <= 30)
    {
      digitalWrite(dir1, HIGH);
      for(int x = 0; x < (51.2*USMin); x++)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
      }
    }  

    if(USMin > 30)
    {
      digitalWrite(dir1, LOW);
      for(int x = 0; x < (51.2*(60-USMin)); x++)
      {
        digitalWrite(step1, HIGH);
        delayMicroseconds(1000);
        digitalWrite(step1, LOW);
        delayMicroseconds(1000);
      }  
    }
    stepmin = USMin;
  }

  if(stephr != USHr)
  {
    digitalWrite(MS1, LOW);
    digitalWrite(MS2, LOW);
    digitalWrite(MS3, LOW);

    if((USHr > 0 && USHr <= 6) || (USHr > 12 && USHr <= 18))
    {
      digitalWrite(dir2, HIGH);
      if(USHr > 0 && USHr <= 6)
      {
        for(int x = 0; x < (256*USHr); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }
      }
      else
      {
        for(int x = 0; x < (256*(USHr-12)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }  
      }
    }

    if((USHr > 6 && USHr < 12) || (USHr > 18 && USHr < 24))
    {
      digitalWrite(dir2, LOW);
      if(USHr > 6 && USHr < 12)
      {
        for(int x = 0; x < (256*(12-USHr)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }
      }
      else
      {
        for(int x = 0; x < (256*(24-USHr)); x++)
        {
          digitalWrite(step2, HIGH);
          delayMicroseconds(1000);
          digitalWrite(step2, LOW);
          delayMicroseconds(1000);
        }  
      }
    }  
    stephr = USHr;
  }
}

void Neo_IN()                                                 //Function for the neopixel to show the daylight
{
  if(HR >=6 && HR <= 9)
  {
    for(int i=0;i<19;i++)
    {
      pixels.setPixelColor(i, pixels.Color( 174, 214, 241 )); // Light blue color.
      pixels.show(); // This sends the updated pixel color to the hardware.
    }
  }
  
  if(HR >=10 && HR <= 12)
  {
    for(int i=0;i<19;i++)
    {
      pixels.setPixelColor(i, pixels.Color( 249, 231, 159 )); // Yellow color.
      pixels.show(); // This sends the updated pixel color to the hardware.
    }
  
  }  

  if(HR >=13 && HR <= 16)
  {
    for(int i=0;i<19;i++)
    {
      pixels.setPixelColor(i, pixels.Color( 250, 215, 160 )); // Orange color.
      pixels.show(); // This sends the updated pixel color to the hardware.
    }
  }  

  if(HR >=17 && HR <= 19)
  {
    for(int i=0;i<19;i++)
    {
      pixels.setPixelColor(i, pixels.Color(247,220,143)); // Moderately dark blue color.
      pixels.show(); // This sends the updated pixel color to the hardware.
    }
  }  

  if(HR >=20 && HR <= 22)
  {
    for(int i=0;i<19;i++)
    {
      pixels.setPixelColor(i, pixels.Color( 36, 113, 163 )); // Dark blue color.
      pixels.show(); // This sends the updated pixel color to the hardware.
    }
  }
  
  else
  {
    pixels.clear();  
  }
}

void Presence()                                               //Function for the ultrasonic sensor
{
  long duration, distance;

  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  duration = pulseIn(echoPin, HIGH);
  distance = (duration/2) / 29.1;
  Serial.println(distance);
  delay(1000);
  
  if(distance > 20 && distance <=50)
  {
    Serial.println("Human within 10cm");
    Show_USTime();
    delay(10000);
    Set_Needle(); 
  }

  else if(distance > 100)
  {
    if(HR < 6 && HR >= 23)
    {
      Serial.println("Human within 30cm");
      Night_light();   
    }
  }  
}

void Night_light()                                         //Function to turn ON the night light
{
  for(int i=0;i<19;i++)
  {
    pixels.setPixelColor(i, pixels.Color(255, 244, 229)); // warm light color.
    pixels.show(); // This sends the updated pixel color to the hardware.
  }
  delay(10000);
  pixels.clear();
}
